@@ -1,8 +1,8 @@
 from django.contrib.auth.decorators import login_required, permission_required
 from django.core.exceptions import ObjectDoesNotExist
 from django.db import models
-from django.http import Http404, HttpResponse, JsonResponse
-from django.shortcuts import render
+from django.http import HttpResponse, HttpResponseNotFound, JsonResponse
+from django.shortcuts import get_object_or_404, render
 from django.utils.translation import gettext_lazy as _
 from django.views.decorators.cache import cache_page
 from eveuniverse.models import EveEntity
@@ -20,7 +20,7 @@ from ..core import BaseConfig, ContactType
 from ..helpers.evecharacter import EveCharacterHelper
 from ..helpers.evecorporation import EveCorporation
 from ..helpers.writers import UnicodeWriter
-from ..models import ContactSet, StandingRequest, StandingRevocation
+from ..models import ContactSet, RequestLogEntry, StandingRequest, StandingRevocation
 from .helpers import DEFAULT_ICON_SIZE, add_common_context
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
@@ -62,7 +62,6 @@ def view_pilots_standings_json(request):
         contacts = ContactSet.objects.latest()
     except ContactSet.DoesNotExist:
         contacts = ContactSet()
-
     character_contacts_qs = (
         contacts.contacts.filter_characters()
         .select_related(
@@ -147,7 +146,6 @@ def download_pilot_standings(request):
         contacts = ContactSet.objects.latest()
     except ContactSet.DoesNotExist:
         contacts = ContactSet()
-
     writer.writerow(
         [
             "character_id",
@@ -179,16 +177,16 @@ def download_pilot_standings(request):
         state = ""
         try:
             ownership = CharacterOwnership.objects.get(character=char)
+        except CharacterOwnership.DoesNotExist:
+            main_character_name = ""
+            main = None
+        else:
             state = ownership.user.profile.state.name
             main = ownership.user.profile.main_character
             if main is None:
                 main_character_name = ""
             else:
                 main_character_name = main.character_name
-        except CharacterOwnership.DoesNotExist:
-            main_character_name = ""
-            main = None
-
         pilot = [
             pilot_standing.eve_entity_id,
             pilot_standing.eve_entity.name,
@@ -204,7 +202,6 @@ def download_pilot_standings(request):
             pilot_standing.standing,
             ", ".join([label.name for label in pilot_standing.labels.all()]),
         ]
-
         writer.writerow([str(v) if v is not None else "" for v in pilot])
     return response
 
@@ -229,7 +226,6 @@ def view_groups_standings(request):
 
     else:
         groups_count = None
-
     context = {
         "lastUpdate": last_update,
         "organization": organization,
@@ -250,7 +246,6 @@ def view_groups_standings_json(request):
         contacts = ContactSet.objects.latest()
     except ContactSet.DoesNotExist:
         contacts = ContactSet()
-
     corporations_qs = (
         contacts.contacts.filter_corporations()
         .select_related(
@@ -333,7 +328,6 @@ def view_groups_standings_json(request):
                 "main_character_icon_url": main_character_icon_url,
             }
         )
-
     alliances_data = list()
     for contact in (
         contacts.contacts.filter_alliances()
@@ -350,7 +344,6 @@ def view_groups_standings_json(request):
                 "labels": [label.name for label in contact.labels.all()],
             }
         )
-
     my_groups_data = {"corps": corporations_data, "alliances": alliances_data}
     return JsonResponse(my_groups_data, safe=False)
 
@@ -485,7 +478,6 @@ def _compose_standing_requests_data(
                     user=req.user, quick_check=quick_check
                 )
             )
-
         else:
             contact_name = ""
             contact_icon_url = ""
@@ -506,7 +498,6 @@ def _compose_standing_requests_data(
             labels = []
         else:
             labels = [obj.name for obj in my_contact.labels.all()]
-
         requests_data.append(
             {
                 "contact_id": req.contact_id,
@@ -533,7 +524,6 @@ def _compose_standing_requests_data(
                 "action_by": req.action_by.username if req.action_by else "(System)",
             }
         )
-
     return requests_data
 
 
@@ -546,15 +536,18 @@ def manage_requests_write(request, contact_id):
         actioned = 0
         for r in StandingRequest.objects.filter(contact_id=contact_id):
             r.mark_actioned(request.user)
+            RequestLogEntry.objects.create_from_standing_request(
+                r, RequestLogEntry.Action.CONFIRMED, request.user
+            )
             actioned += 1
         if actioned > 0:
             return HttpResponseNoContent()
-        return Http404()
+        return HttpResponseNotFound()
     elif request.method == "DELETE":
-        try:
-            standing_request = StandingRequest.objects.get(contact_id=contact_id)
-        except StandingRequest.DoesNotExist:
-            return Http404()
+        standing_request = get_object_or_404(StandingRequest, contact_id=contact_id)
+        RequestLogEntry.objects.create_from_standing_request(
+            standing_request, RequestLogEntry.Action.REJECTED, request.user
+        )
         standing_request.delete()
         if SR_NOTIFICATIONS_ENABLED:
             entity_name = EveEntity.objects.resolve_name(contact_id)
@@ -566,7 +559,7 @@ def manage_requests_write(request, contact_id):
             notify(user=standing_request.user, title=title, message=message)
 
         return HttpResponseNoContent()
-    return Http404()
+    return HttpResponseNotFound()
 
 
 @login_required
@@ -574,39 +567,42 @@ def manage_requests_write(request, contact_id):
 def manage_revocations_write(request, contact_id):
     contact_id = int(contact_id)
     logger.debug(
-        "manage_revocations_write called by %s for contact_id %d",
-        request.user,
+        "manage_revocations_write called by %s for contact_id %s",
+        str(request.user),
         contact_id,
     )
     if request.method == "PUT":
         actioned = 0
-        for r in StandingRevocation.objects.filter(contact_id=contact_id):
+        for r in StandingRevocation.objects.filter(
+            contact_id=contact_id, action_date__isnull=True
+        ):
             r.mark_actioned(request.user)
+            RequestLogEntry.objects.create_from_standing_request(
+                r, RequestLogEntry.Action.CONFIRMED, request.user
+            )
             actioned += 1
         if actioned > 0:
             return HttpResponseNoContent()
-        else:
-            return Http404
+        return HttpResponseNotFound
     elif request.method == "DELETE":
-        try:
-            standing_revocation = StandingRevocation.objects.get(contact_id=contact_id)
-        except StandingRevocation.DoesNotExist:
-            return Http404()
-        else:
-            StandingRevocation.objects.filter(contact_id=contact_id).delete()
-            if SR_NOTIFICATIONS_ENABLED and standing_revocation.user:
-                entity_name = EveEntity.objects.resolve_name(contact_id)
-                title = _("Standing revocation for %s rejected" % entity_name)
-                message = _(
-                    "Your standing revocation for '%s' "
-                    "has been rejected by %s." % (entity_name, request.user)
-                )
-                notify(user=standing_revocation.user, title=title, message=message)
-
-            return HttpResponseNoContent()
-
-    else:
-        return Http404()
+        standing_revocations_qs = StandingRevocation.objects.filter(
+            contact_id=contact_id
+        )
+        standing_revocation = standing_revocations_qs.first()
+        RequestLogEntry.objects.create_from_standing_request(
+            standing_revocation, RequestLogEntry.Action.REJECTED, request.user
+        )
+        standing_revocations_qs.delete()
+        if SR_NOTIFICATIONS_ENABLED and standing_revocation.user:
+            entity_name = EveEntity.objects.resolve_name(contact_id)
+            title = _("Standing revocation for %s rejected" % entity_name)
+            message = _(
+                "Your standing revocation for '%s' "
+                "has been rejected by %s." % (entity_name, request.user)
+            )
+            notify(user=standing_revocation.user, title=title, message=message)
+        return HttpResponseNoContent()
+    return HttpResponseNotFound()
 
 
 ###################
